// ------------------------------------------------------------------------------------------------

/*
    This file is part of Style Sheet Chooser Plus.

    Style Sheet Chooser Plus is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Style Sheet Chooser Plus is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Style Sheet Chooser Plus; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// ------------------------------------------------------------------------------------------------
//
// Style Sheet Chooser Plus
//
// Copyright 2004-2006, OLab (http://www.olab.info/)
// Thanks to Rob Tougher (http://www.robtougher.com) for the original extension
// namely Stylesheet Chooser, on which this one is partly based
//
// This extension persists your choices for alternate stylesheets on
// a per-domain basis. For example, if you browse to http://domain.com/index1.html,
// choose the alternate stylesheet "Alt", and browse to http://domain.com/index2.html,
// the alternate stylesheet "Alt" will be used on the second page, index2.html.
// Furthermore, an icon in the status bar helps you to select it.
//
// Choices are persisted between window instances, so your choices will be remembered
// if you close Firefox and restart it.
//
// ------------------------------------------------------------------------------------------------

var SSCP = {
	/* Initializes the extension */
	init: function() {
		// The status button dom element
		this.pageStyleButton = null;
		
		// We use SQLite to save and load the cache to/from disk and for preferences
		this.prefService = new SSCPSQLite();
		
		// Locale strings
		const STRBUNDLE = Components.classes['@mozilla.org/intl/stringbundle;1']
                        .getService(Components.interfaces.nsIStringBundleService);
        this.strings = STRBUNDLE.createBundle('chrome://stylesheetchooser_plus/locale/sscp.properties');
		
		// Import old caches end preferences if this is the first run of this new version
		var firstRun = this.prefService.getValue(this.prefService.optionsTable, 'firstRun', true);
		if (firstRun) {
			var imported = SSCPLegacy.run();
			if (imported) this.prefService.setValue(this.prefService.optionsTable, 'firstRun', 'false');
		}
		
		// Get propagation preference
		this.propagation = this.prefService.getValue(this.prefService.optionsTable, 'propagation', true);

		// Get the theme status button default state
		this.showIcon = this.prefService.getValue(this.prefService.optionsTable, 'alwaysShowStatusButton', true);
		
		// We use the observer service to synchronize style selection between
		// windows, if propagation is enabled
		this.observerService = Components.classes["@mozilla.org/observer-service;1"].getService(
								Components.interfaces.nsIObserverService);
		this.observerService.addObserver(SSCP.stylesheetChangeObserver,
						"SSCP.StylesheetChange", false);
		// This flag is used to avoid switching twice the style for the current window
		// (with menu selection and propagation)
        this.notifying = false;
	}
};

/* Adds a stylesheet choice to cache
 * @param string key Unique identifier of the value
 * @param string value The value
 */
SSCP.addToCache = function(key, value) {
	if (key != 'key_undefined')
		this.prefService.setValue(this.prefService.authorStylesTable, key, value);
}
/* Gets a value from th cache
 * @param string key Unique identifier of the value
 * @return string The value corresponding to key or empty string if it doesn't exist
 */
SSCP.getCacheValue = function(key) {
	return this.prefService.getValue(this.prefService.authorStylesTable, key, '');
}
/* Checks value presence in the cache
 * @param string key Unique identifier of the value
 * @return boolean Presence or not of a key,value pair
 */
SSCP.keyExistsInCache = function (key) {
	var hasStylesheet = false;
	if (this.prefService.getValue(this.prefService.authorStylesTable, key, false) != false)
		hasStylesheet = true;
	
	return hasStylesheet;
}
/* Removes a value from the cache
 * @param string key Unique identifier of the value
 */
SSCP.removeFromCache = function(key) {
	this.prefService.delValue(this.prefService.authorStylesTable, key);
}
/* Builds a key for the current browser
 * @param object The current browser tab
 * @return string The key, calculated from tab's location
 */
SSCP.makeCacheKey = function(browserObject) {
	var key = "key_undefined";
	try {
		var location = browserObject.contentDocument.location.toString();
		if (location.indexOf('file://') != -1)
			key = location;
		else
			key = browserObject.contentDocument.location.hostname;
	} catch(e) {}
	
	return key;
}
/* An object which observes stylesheet changes */
SSCP.stylesheetChangeObserver = {
	/* Trigger activated when this observer is notified
	 * @param object subject
	 * @param string sTopic The event topic
	 * @param string sData Data passed by the notifier (here, the stylesheet's name)
	 */
	observe: function(subject, sTopic, sData)
	{
		if (sTopic == "SSCP.StylesheetChange") {
			if (typeof(SSCP) != 'undefined') {
				 if (!SSCP.notifying) {
		            var keyTmp = SSCP.makeCacheKey(document.getElementById("content").mCurrentBrowser);
		        	if (keyTmp == sData) {// If the site is the same that the one notifying
		                var cacheValue = SSCP.getCacheValue(sData);
		                stylesheetSwitchAll(window.content, cacheValue);
		                // Reactivate styles if disabled
		                setStyleDisabled(false);
		        	}
				}
				SSCP.notifying = false;
			}
		}
	}
};

/* Notifies observers of a stylesheet change
 * @param string key Unique identifier of the current tab
 */
SSCP.fireStylesheetChange = function (key) {
	this.observerService.notifyObservers(null, "SSCP.StylesheetChange", key);
}

/* Changes the currently activated alternate stylesheet, regarding cache value */
SSCP.doUpdateSSCP = function() {

    // Get cache value
	var key = SSCP.makeCacheKey(document.getElementById("content").mCurrentBrowser);
	var cacheValue = SSCP.getCacheValue(key);

	// Change the active stylesheet according to cache
	if (cacheValue != '') {
		if (cacheValue != '_nostyle') {
			stylesheetSwitchAll(window.content, cacheValue);
	    	setStyleDisabled(false);
	    } else { // Deactivate all styles if necessary
	    	setStyleDisabled(true);
	    }
	}
}

/* Progress listener code.
 * Note: I call too many times the update function (via DOMLinkAdded listener and onStateChange),
 * which must slow down the browser, but I don't know how to polish all of this and keep
 * the "maximal" experience (no stylesheet flickering)   
 */
SSCP.progressListener =
{
	onLocationChange : function(aWebProgress, aRequest, aLocation)
	{
		try {
		    var currentBrowser = document.getElementById("content").selectedBrowser;
			var sLocation = currentBrowser.contentDocument.location;
		    if (sLocation != "about:blank")
                currentBrowser.addEventListener("DOMLinkAdded", SSCP.doUpdateSSCP, false);
			
			// Reactivate styles if we are leaving a website and styles are disabled (only if preferences tell so)
			var keepDisabledState = SSCP.prefService.getValue(SSCP.prefService.optionsTable, 'keepDisabledState', false);
			if (!keepDisabledState) {
				var hostname = sLocation.hostname;
				if (SSCP.prevHost != hostname) {
					setStyleDisabled(false);
					SSCP.prevHost = hostname;
				}
			}
			
            /* Update the stylesheet now, because when a tab is loaded in background and selected,
             * the stylesheet is not updated. */             
            if (SSCP.propagation) SSCP.doUpdateSSCP();
		} catch(e) {
			// For mouse gestures error.
		}
	},

	onProgressChange : function(aWebProgress, aRequest, aCurSelfProgress,
				aMaxSelfProgress, aCurTotalProgress, aMaxTotalProgress)
	{
	},

	onSecurityChange : function(aWebProgress, aRequest, state) {
	},

	onStateChange : function(aWebProgress, aRequest, aStateFlags, aStatus)
	{
		try {
		  // If the page is fully loaded, we can remove the DOMLinkAdded listener
		  if ((aStateFlags & Components.interfaces.nsIWebProgressListener.STATE_STOP) == Components.interfaces.nsIWebProgressListener.STATE_STOP) {
		      var currentBrowser = document.getElementById("content").selectedBrowser;
		      currentBrowser.removeEventListener("DOMLinkAdded", SSCP.doUpdateSSCP, false);
          }
          SSCP.doUpdateSSCP();

		} catch (e) {
			// For mouse gestures error.
		}
	},

	onStatusChange : function(aWebProgress, aRequest, aStatus, message)
	{
	},

	QueryInterface : function(aIID)
	{
		if (aIID.equals(Components.interfaces.nsIWebProgressListener) ||
			aIID.equals(Components.interfaces.nsISupportsWeakReference) ||
			aIID.equals(Components.interfaces.nsIXULBrowserWindow) ||
			aIID.equals(Components.interfaces.nsISupports))
			return this;
		throw Components.results.NS_NOINTERFACE;
	},

	// See bug 123340 (http://bugzilla.mozilla.org/show_bug.cgi?id=123340)
	// This function is not part of the progress listener interface, but the
	// tab browser calls it anyway.
	onLinkIconAvailable : function(a, b)
	{
	}

}


/* Event handler for the window's load event. */
SSCP.windowLoad = function() {
	// Get the page theme status button and show it if preferences item is on
	SSCP.pageStyleButton = document.getElementById("page-theme-button");
	if (SSCP.showIcon) SSCP.pageStyleButton.style.setProperty("display", "list-item", "important");

	/* Keep one progress listener per browser. Any time a location
	 change happens, update the stylesheet */
	document.getElementById("content").addProgressListener(
			SSCP.progressListener.QueryInterface(Components.interfaces.nsIWebProgressListener),
			Components.interfaces.nsIWebProgress.NOTIFY_ALL);

	/* Deleting the default View > Page Style > menupopup */
	var pageStyleMenu = document.getElementById('pageStyleMenu');
	for (var i=0; i<pageStyleMenu.childNodes.length; i++) {
		var child = pageStyleMenu.childNodes[i];
		if (child.id != 'use-style-menu-popup')
			pageStyleMenu.removeChild(child);
	}
	
	/* Change Firefox core code to re-activate updatePageStyle (browser.js)
	 * We need to use setTimeout, otherwise XULBrowserWindow is not available during the call
	 */
	setTimeout(function() {window.XULBrowserWindow.asyncUpdateUI = function () {
		FeedHandler.updateFeeds();
		BrowserSearch.updateSearchButton();
		// Add the call to customized version of updatePageStyle
		SSCP.updatePageStyles();
	}},0);
	
}

SSCP.onMenuClick = function(event) {
	// Get the latest propagation pref
	this.propagation = this.prefService.getValue(this.prefService.optionsTable, 'propagation', false);

	// Get style title
	var sTitle = event.target.getAttribute('data');

	/* If we choose the default style, we don't need to save it,
	 * because the next time the page will be loaded, it will be
	 * automatically selected. Also, we remove it from cache.
	 */
	var key = this.makeCacheKey(document.getElementById("content").mCurrentBrowser);
	if ((sTitle == '') || (sTitle == window.content.document.preferredStylesheetSet))
		this.removeFromCache(key);
	else
		this.addToCache(key, sTitle); // Add the choice to cache
	
	// Update others windows if necessary
	if (this.propagation) {
        /* Propagate to other window instances (we don't need to propagate to other tabs
         * because the OnLocationChange event in WebProgressListener do it automatically) */
        this.notifying = true;
        this.fireStylesheetChange(key);
	}
}

SSCP.onPopupShowing = function (event) {
	var menu = event.target;
	
	var separator = document.createElement("menuseparator");
	menu.appendChild(separator);
	
	var optionsItem = document.createElement("menuitem");
	optionsItem.setAttribute("label", this.strings.GetStringFromName('menuOptionsItem.label'));
	optionsItem.setAttribute("accesskey", this.strings.GetStringFromName('menuOptionsItem.accesskey'));
	optionsItem.setAttribute("tooltiptext", this.strings.GetStringFromName('menuOptionsItem.tooltiptext'));
	menu.appendChild(optionsItem);
	optionsItem.addEventListener("command",
				function (event) {
					window.openDialog('chrome://stylesheetchooser_plus/content/SSCPOptions.xul', '', 'chrome,titlebar,toolbar,centerscreen,modal');
					/* We need to stop propagation, because the command event will be
					 * propagated to menupopup element, which intend to change current stylesheet
					 */
					event.stopPropagation();
				},
				true);
}




// ------------------------------------------------------------------------------------------------
//
// Function to show or hide the statusbar icon
// (Code from original Firefox browser.js file, with some changes)
// ------------------------------------------------------------------------------------------------

SSCP.updatePageStyles = function(evt) {
	var hasAltSS = false;
	var stylesheets = window.content.document.styleSheets;
	var preferredSheet = window.content.document.preferredStylesheetSet;
	for (var i = 0; i < stylesheets.length; ++i) {
		var currentStyleSheet = stylesheets[i];

		if (currentStyleSheet.title && currentStyleSheet.title != preferredSheet) {
			// Skip any stylesheets that don't match the screen media type.
			var media = currentStyleSheet.media.mediaText.toLowerCase();
			if (media && (media.indexOf("screen") == -1) && (media.indexOf("all") == -1))
				continue;
			hasAltSS = true;
			break;
		}
	}

	if (hasAltSS) {
		this.pageStyleButton.setAttribute("themes", "true");
	} else {
		this.pageStyleButton.removeAttribute("themes");
		// Show the theme status button if necessary
		var showIcon = this.prefService.getValue(this.prefService.optionsTable, 'alwaysShowStatusButton', true);
		if (showIcon) this.pageStyleButton.style.setProperty("display", "list-item", "important");
		else if (!showIcon) this.pageStyleButton.style.removeProperty("display");
	}
}

/* Calls initialization code for the extensions */
SSCP.init();
